/*!
  \file
  \brief Small, common, generic utilities go here
*/

#pragma once

#include "gBase.h"

//-------------------------------------------------------------------
// templated misc functions

template<class Type> inline void gSwap(Type &a,Type &b)
{
	Type c = a; a = b; b = c;
}

template<class Type,class Type1,class Type2> inline Type gClamped(const Type &x,const Type1 &lo,const Type2 &hi)
{
	return ( ( x < lo ) ? lo : ( x > hi ) ? hi : x );
}

//-------------------------------------------------------------------
//! handy little type for doing compile-time switches in meta-programming
//	gBoolAsType_True and gBoolAsType_False

template <bool t_bool>
struct gBoolToType
{
	enum { value = t_bool };
};

typedef gBoolToType<true>	gBoolAsType_True;
typedef gBoolToType<false>	gBoolAsType_False;

//-------------------------------------------------------------------
// Int math :

inline bool gIsPow2(const int x)
{
	return ! (x & ~(-x));
}

inline int gNextPow2(const int x)
{
	// @@ could be faster, but who cares?
	int y = 1;
	while ( y < x )
	{
		y += y;
	}
	return y;
}

// Aligns v up, alignment must be a power of two.
inline int gAlignUp(const int v, const int alignment)
{
	ASSERT(gIsPow2(alignment));
	ASSERT( v >= 0 );
	return (v+(alignment-1)) & ~(alignment-1);
}

// Aligns v down, alignment must be a power of two.
inline int gAlignDown(const int v, const int alignment)
{
	ASSERT(gIsPow2(alignment));
	ASSERT( v >= 0 );
	return v & ~(alignment-1);
}

//-------------------------------------------------------------------
// crazy macros : DO_ONCE and AT_STARTUP

#define STRING_JOIN(arg1, arg2) STRING_JOIN_DELAY(arg1, arg2)
#define STRING_JOIN_DELAY(arg1, arg2) DO_STRING_JOIN(arg1, arg2)
#define DO_STRING_JOIN(arg1, arg2) arg1 ## arg2

// Macro for static-initialization one-liners.
// !! Only works in statically-linked .obj files. !!
#define AT_STARTUP(some_code)	\
namespace { static struct STRING_JOIN(AtStartup_, __LINE__) { STRING_JOIN(AtStartup_, __LINE__)() { some_code; } } STRING_JOIN(AtStartup_, __LINE__) ## Instance; };


#define DO_ONCE(some_code)	\
do { static bool once = true; if ( once ) { once = false; { some_code; } } } while(0)

//-------------------------------------------------------------------
// names that don't conflict with standards :

#define gMIN(a,b)	( ((a) < (b)) ? (a) : (b) )
#define gMAX(a,b)	( ((a) < (b)) ? (b) : (a) )

#define gMIN3(a,b,c)	gMIN(gMIN(a,b),c)
#define gMAX3(a,b,c)	gMAX(gMAX(a,b),c)

#define gABS(a)		( ((a) < 0) ? -(a) : (a) )

//-------------------------------------------------------------------
// to hell with conflicts, I like the standard names :
//	(this requires you to go into your STL headers and
//	rip out the damned std::min and std::max)
/*
#ifdef min
#undef min
#endif
#ifdef max
#undef max
#endif

#define min gMIN
#define max gMAX
#define min3 gMIN3
#define max3 gMAX3
*/
//-------------------------------------------------------------------
